<?php

namespace Drupal\choices_autocomplete\Plugin\Field\FieldWidget;

use Drupal\choices_autocomplete\ChoicesAutocompleteDefaults;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides Choices.js autocomplete widget trait.
 */
trait ChoicesWidgetTrait {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    return ['options' => ChoicesAutocompleteDefaults::getOptions()] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function getSetting($key) {
    $setting = parent::getSetting($key);
    if (is_array($setting)) {
      $defaultOptions = static::defaultSettings()['options'];
      return NestedArray::mergeDeep($defaultOptions, $setting);
    }
    return $setting;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $form = parent::settingsForm($form, $form_state);
    $form['placeholder']['#access'] = $form['size']['#access'] = FALSE;

    $options = $this->getSetting('options');
    $form['options'] = ['#tree' => TRUE];
    $form['options']['plugin']['searchPlaceholderValue'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Start typing text'),
      '#description' => $this->t('Text shown as search input placeholder.'),
      '#default_value' => $options['plugin']['searchPlaceholderValue'],
    ];
    $form['options']['plugin']['loadingText'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Searching text'),
      '#description' => $this->t('Text shown in dropdown as search results are being retrieved.'),
      '#default_value' => $options['plugin']['loadingText'],
    ];
    $form['options']['plugin']['noResultsText'] = [
      '#type' => 'textfield',
      '#title' => $this->t('No results text'),
      '#description' => $this->t('Text shown in dropdown when no results were found.'),
      '#default_value' => $options['plugin']['noResultsText'],
    ];
    $form['options']['plugin']['itemSelectText'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Press to select text'),
      '#description' => $this->t('Text shown in dropdown next to the current highlighted result.'),
      '#default_value' => $options['plugin']['itemSelectText'],
    ];
    $form['options']['plugin']['maxItemText'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Max items text'),
      '#description' => $this->t('Text shown in dropdown when the maximum number of items are selected.'),
      '#default_value' => $options['plugin']['maxItemText'],
    ];
    $form['options']['instance']['remove_item_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Remove item text'),
      '#description' => $this->t('Text shown in button to remove a selected item.'),
      '#default_value' => $options['instance']['remove_item_text'],
    ];
    $form['options']['plugin']['position'] = [
      '#type' => 'select',
      '#title' => $this->t('Dropdown position'),
      '#description' => $this->t('Change the search results dropdown position.'),
      '#options' => [
        'auto' => $this->t('Automatic (recommended)'),
        'top' => $this->t('Top'),
        'bottom' => $this->t('Bottom'),
      ],
      '#default_value' => $options['plugin']['position'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $cardinality = $items->getFieldDefinition()->getFieldStorageDefinition()->getCardinality();

    $element += [
      '#type' => 'choices_autocomplete',
      '#multiple' => $cardinality === FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED || $cardinality > 1,
      '#cardinality' => $cardinality,
      '#choices_autocomplete_options' => $this->getSetting('options'),
      '#default_value' => [],
    ];

    return $element;
  }

}
